﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using WebGraphic.Models;
using PagedList;
using System.Threading.Tasks;
using System.Net;
using Microsoft.Owin.Security;
using Microsoft.AspNet.Identity;
using Microsoft.AspNet.Identity.Owin;

namespace WebGraphic.Controllers
{
    [Authorize]
    public class ArticleController : Controller
    {
        WGDb _db = new WGDb();
        public ActionResult Index([Bind(Prefix = "id")] int categoryId)
        {
            var category = _db.Category.Find(categoryId);
            if (category != null)
            {
                return View(category);
            }
            return HttpNotFound();
        }

        [HttpGet]
        [Authorize(Roles = "Admin, Moderator")]
        public ActionResult Create(int categoryId)
        {
            return View();
        }

        [HttpPost]
        [ValidateInput(false)]
        [Authorize(Roles = "Admin, Moderator")]
        public ActionResult Create(Article articles)
        {
            if (ModelState.IsValid)
            {
                articles.CreatedBy = User.Identity.Name;
                articles.Created = DateTime.Now;
                _db.Article.Add(articles);
                _db.SaveChanges();
                return RedirectToAction("Index", new { id = articles.CategoryId });
            }
            return View(articles);
        }

        [HttpGet]
        [Authorize(Roles = "Admin, Moderator")]
        public ActionResult Edit(int id)
        {
            var model = _db.Article.Find(id);
            return View(model);
        }

        [HttpPost]
        [ValidateInput(false)]
        [Authorize(Roles = "Admin, Moderator")]
        public ActionResult Edit(Article articles)
        {
            if (ModelState.IsValid)
            {
                articles.CreatedBy = User.Identity.Name;
                articles.Created = DateTime.Now;
                _db.Entry(articles).State = EntityState.Modified;
                _db.SaveChanges();
                return RedirectToAction("Index", new { id = articles.CategoryId });
            }
            return View(articles);
        }

        // GET: /Article/Details/5
        public async Task<ActionResult> Details(int? id)
        {
           
            Article article = await _db.Article.FindAsync(id);
           
            return View(article);
        }

        // GET: Article/Delete/5
        [Authorize(Roles = "Moderator")]
        public ActionResult Delete(int id = 0)
        {
            Article article = _db.Article.Find(id);
            if (article == null)
            {
                return HttpNotFound();
            }
            return View(article);
        }

        // POST: Article/Delete/5
        [Authorize(Roles = "Moderator")]
        [HttpPost, ActionName("Delete")]
        [ValidateAntiForgeryToken]
        public ActionResult DeleteConfirmed(int id)
        {
            Article article = _db.Article.Find(id);
            _db.Article.Remove(article);
            _db.SaveChanges();
            return RedirectToAction("Index", new { id = article.CategoryId });
        }


        public ActionResult Autocomplete(string term)
        {
            var model =
                _db.Article
                   .Where(c => c.Title.StartsWith(term))
                   .Take(10)
                   .Select(c => new
                   {
                       label = c.Title
                   });

            return Json(model, JsonRequestBehavior.AllowGet);
        }

        public ActionResult ArticleList(string searchterm = null, int page = 1)
        {
            var model =
                _db.Article
                .OrderByDescending(c => c.Id)
                .Where(c => searchterm == null || c.Title.StartsWith(searchterm))
                .Select(c => new ArticleListViewModel
                {
                    Id = c.Id,
                    Title = c.Title
                }).ToPagedList(page, 10);

            if (Request.IsAjaxRequest())
            {
                return PartialView("_Articles", model);
            }

            return View(model);
        }

        protected override void Dispose(bool disposing)
        {
            _db.Dispose();
            base.Dispose(disposing);
        }
    }
}