﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using WebGraphic.Models;
using PagedList;


namespace WebGraphic.Controllers
{
    [Authorize]
    public class CategoryController : Controller
    {
        private WGDb db = new WGDb();
        [Authorize(Roles = "Admin, Moderator")]
        public ActionResult Index()
        {
            return View(db.Category.ToList());
        }

        // GET: Categories/Create
        [Authorize(Roles = "Admin, Moderator")]
        public ActionResult Create()
        {
            return View();
        }

        [Authorize(Roles = "Admin, Moderator")]
        [HttpPost]
        public ActionResult Create(Category category)
        {
            if (ModelState.IsValid)
            {
                db.Category.Add(category);
                db.SaveChanges();
                return RedirectToAction("Index");
            }

            return View(category);
        }

        // GET: Categories/Edit/5
        [Authorize(Roles = "Admin, Moderator")]
        public ActionResult Edit(int id = 0)
        {
            Category category = db.Category.Find(id);
            if (category == null)
            {
                return HttpNotFound();
            }
            return View(category);
        }

        [HttpPost]
        [Authorize(Roles = "Admin, Moderator")]
        public ActionResult Edit(Category category)
        {
            if (ModelState.IsValid)
            {
                db.Entry(category).State = EntityState.Modified;
                db.SaveChanges();
                return RedirectToAction("Index");
            }
            return View(category);
        }

        // GET: Categories/Delete/5
        [Authorize(Roles = "Admin, Moderator")]
        public ActionResult Delete(int id = 0)
        {
            Category category = db.Category.Find(id);
            if (category == null)
            {
                return HttpNotFound();
            }
            return View(category);
        }

        // POST: Categories/Delete/5
        [Authorize(Roles = "Admin, Moderator")]
        [HttpPost, ActionName("Delete")]
        [ValidateAntiForgeryToken]
        public ActionResult DeleteConfirmed(int id)
        {
            Category category = db.Category.Find(id);
            db.Category.Remove(category);
            db.SaveChanges();
            return RedirectToAction("Index");
        }

        public ActionResult Autocomplete(string term)
        {
            var model =
                db.Category
                   .Where(c => c.CategoryName.StartsWith(term))
                   .Take(10)
                   .Select(c => new
                   {
                       label = c.CategoryName
                   });

            return Json(model, JsonRequestBehavior.AllowGet);
        }

        public ActionResult CategoryList(string searchterm = null, int page = 1)
        {
            var model =
                db.Category
                .OrderByDescending(c => c.Id)
                .Where(c => searchterm == null || c.CategoryName.StartsWith(searchterm))
                .Select(c => new CategoryListViewModel
                {
                    Id = c.Id,
                    Name = c.CategoryName
                }).ToPagedList(page, 10);

            if (Request.IsAjaxRequest())
            {
                return PartialView("_Categories", model);
            }

            return View(model);
        }

        protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }
    }
}