﻿using Microsoft.AspNet.Identity;
using Microsoft.AspNet.Identity.EntityFramework;
using System.ComponentModel.DataAnnotations;
using System.Data.Entity;
using System.Security.Claims;
using System.Threading.Tasks;

namespace WebGraphic.Models
{
    public class ApplicationUser : IdentityUser
    {
        [Display(Name = "Jméno")]
        public string FirstName { get; set; }
        [Display(Name = "Příjmení")]
        public string LastName { get; set; }
        [DisplayFormat(DataFormatString = "{0:dd/MM/yyyy}", ApplyFormatInEditMode = true)]
        [Display(Name = "Datum narození")]
        public System.DateTime? BirthDate { get; set; }
        public async Task<ClaimsIdentity> GenerateUserIdentityAsync(UserManager<ApplicationUser> manager)
        {
            var userIdentity = await manager.CreateIdentityAsync(this, DefaultAuthenticationTypes.ApplicationCookie);
            return userIdentity;
        }
    }

    public class WGDb : IdentityDbContext<ApplicationUser>
    {
        public WGDb()
            : base("DefaultConnection", throwIfV1Schema: false)
        {
        }

        static WGDb()
        {
            // Seed the database with admin user credentials and admin role
            Database.SetInitializer<WGDb>(new ApplicationDbInitializer());
        }

        public static WGDb Create()
        {
            return new WGDb();
        }

        public DbSet<Article> Article { get; set; }

        public DbSet<Category> Category { get; set; }
    }
}